@@ -338,7 +338,9 @@ def _compare_columns(
 
 class _constraint_sig(object):
     def md_name_to_sql_name(self, context):
-        return self.name
+        return sqla_compat._get_constraint_final_name(
+            self.const, context.dialect
+        )
 
     def __eq__(self, other):
         return self.const == other.const
@@ -529,7 +531,8 @@ def _compare_indexes_and_uniques(
     metadata_names = dict(
         (c.md_name_to_sql_name(autogen_context), c)
         for c in metadata_unique_constraints.union(metadata_indexes)
-        if c.name is not None
+        if isinstance(c, _ix_constraint_sig)
+        or sqla_compat._constraint_is_named(c.const, autogen_context.dialect)
     )
 
     conn_uniques_by_name = dict((c.name, c) for c in conn_unique_constraints)
@@ -556,7 +559,11 @@ def _compare_indexes_and_uniques(
     )
     metadata_indexes_by_sig = dict((ix.sig, ix) for ix in metadata_indexes)
     unnamed_metadata_uniques = dict(
-        (uq.sig, uq) for uq in metadata_unique_constraints if uq.name is None
+        (uq.sig, uq)
+        for uq in metadata_unique_constraints
+        if not sqla_compat._constraint_is_named(
+            uq.const, autogen_context.dialect
+        )
     )
 
     # assumptions:
@@ -809,7 +816,7 @@ def _setup_autoincrement(
 
     if metadata_col.table._autoincrement_column is metadata_col:
         alter_column_op.kw["autoincrement"] = True
-    elif util.sqla_110 and metadata_col.autoincrement is True:
+    elif metadata_col.autoincrement is True:
         alter_column_op.kw["autoincrement"] = True
     elif metadata_col.autoincrement is False:
         alter_column_op.kw["autoincrement"] = False
@@ -873,7 +880,10 @@ def _render_server_default_for_compare(
             metadata_default = metadata_default.arg
         else:
             metadata_default = str(
-                metadata_default.arg.compile(dialect=autogen_context.dialect)
+                metadata_default.arg.compile(
+                    dialect=autogen_context.dialect,
+                    compile_kwargs={"literal_binds": True},
+                )
             )
     if isinstance(metadata_default, compat.string_types):
         if metadata_col.type._type_affinity is sqltypes.String:
