@@ -1,36 +1,41 @@
+import logging
 import re
 
-from ..util import compat
-from .. import util
-from .base import compiles, alter_column, alter_table, format_table_name, \
-    format_type, AlterColumn, RenameTable
-from .impl import DefaultImpl
-from sqlalchemy.dialects.postgresql import INTEGER, BIGINT
-from ..autogenerate import render
-from sqlalchemy import text, Numeric, Column
-from sqlalchemy.sql.expression import ColumnClause
-from sqlalchemy.types import NULLTYPE
+from sqlalchemy import Column
+from sqlalchemy import Numeric
+from sqlalchemy import text
 from sqlalchemy import types as sqltypes
-
-from ..operations.base import Operations
-from ..operations.base import BatchOperations
-from ..operations import ops
-from ..util import sqla_compat
-from ..operations import schemaobj
-
-import logging
-
+from sqlalchemy.dialects.postgresql import BIGINT
+from sqlalchemy.dialects.postgresql import ExcludeConstraint
+from sqlalchemy.dialects.postgresql import INTEGER
+from sqlalchemy.sql.expression import ColumnClause
 from sqlalchemy.sql.expression import UnaryExpression
+from sqlalchemy.types import NULLTYPE
 
-if util.sqla_100:
-    from sqlalchemy.dialects.postgresql import ExcludeConstraint
+from .base import alter_column
+from .base import alter_table
+from .base import AlterColumn
+from .base import ColumnComment
+from .base import compiles
+from .base import format_table_name
+from .base import format_type
+from .base import RenameTable
+from .impl import DefaultImpl
+from .. import util
+from ..autogenerate import render
+from ..operations import ops
+from ..operations import schemaobj
+from ..operations.base import BatchOperations
+from ..operations.base import Operations
+from ..util import compat
+from ..util import sqla_compat
 
 
 log = logging.getLogger(__name__)
 
 
 class PostgresqlImpl(DefaultImpl):
-    __dialect__ = 'postgresql'
+    __dialect__ = "postgresql"
     transactional_ddl = True
 
     def prep_table_for_batch(self, table):
@@ -38,13 +43,18 @@ class PostgresqlImpl(DefaultImpl):
             if constraint.name is not None:
                 self.drop_constraint(constraint)
 
-    def compare_server_default(self, inspector_column,
-                               metadata_column,
-                               rendered_metadata_default,
-                               rendered_inspector_default):
+    def compare_server_default(
+        self,
+        inspector_column,
+        metadata_column,
+        rendered_metadata_default,
+        rendered_inspector_default,
+    ):
         # don't do defaults for SERIAL columns
-        if metadata_column.primary_key and \
-                metadata_column is metadata_column.table._autoincrement_column:
+        if (
+            metadata_column.primary_key
+            and metadata_column is metadata_column.table._autoincrement_column
+        ):
             return False
 
         conn_col_default = rendered_inspector_default
@@ -56,53 +66,68 @@ class PostgresqlImpl(DefaultImpl):
         if None in (conn_col_default, rendered_metadata_default):
             return not defaults_equal
 
-        if metadata_column.server_default is not None and \
-            isinstance(metadata_column.server_default.arg,
-                       compat.string_types) and \
-                not re.match(r"^'.+'$", rendered_metadata_default) and \
-                not isinstance(inspector_column.type, Numeric):
-                # don't single quote if the column type is float/numeric,
-                # otherwise a comparison such as SELECT 5 = '5.0' will fail
-            rendered_metadata_default = re.sub(
-                r"^u?'?|'?$", "'", rendered_metadata_default)
+        if compat.py2k:
+            # look for a python 2 "u''" string and filter
+            m = re.match(r"^u'(.*)'$", rendered_metadata_default)
+            if m:
+                rendered_metadata_default = "'%s'" % m.group(1)
+
+        # check for unquoted string and quote for PG String types
+        if (
+            not isinstance(inspector_column.type, Numeric)
+            and metadata_column.server_default is not None
+            and isinstance(
+                metadata_column.server_default.arg, compat.string_types
+            )
+            and not re.match(r"^'.*'$", rendered_metadata_default)
+        ):
+            rendered_metadata_default = "'%s'" % rendered_metadata_default
 
         return not self.connection.scalar(
-            "SELECT %s = %s" % (
-                conn_col_default,
-                rendered_metadata_default
-            )
+            "SELECT %s = %s" % (conn_col_default, rendered_metadata_default)
         )
 
-    def alter_column(self, table_name, column_name,
-                     nullable=None,
-                     server_default=False,
-                     name=None,
-                     type_=None,
-                     schema=None,
-                     autoincrement=None,
-                     existing_type=None,
-                     existing_server_default=None,
-                     existing_nullable=None,
-                     existing_autoincrement=None,
-                     **kw
-                     ):
+    def alter_column(
+        self,
+        table_name,
+        column_name,
+        nullable=None,
+        server_default=False,
+        name=None,
+        type_=None,
+        schema=None,
+        autoincrement=None,
+        existing_type=None,
+        existing_server_default=None,
+        existing_nullable=None,
+        existing_autoincrement=None,
+        **kw
+    ):
 
-        using = kw.pop('postgresql_using', None)
+        using = kw.pop("postgresql_using", None)
 
         if using is not None and type_ is None:
             raise util.CommandError(
-                "postgresql_using must be used with the type_ parameter")
+                "postgresql_using must be used with the type_ parameter"
+            )
 
         if type_ is not None:
-            self._exec(PostgresqlColumnType(
-                table_name, column_name, type_, schema=schema,
-                using=using, existing_type=existing_type,
-                existing_server_default=existing_server_default,
-                existing_nullable=existing_nullable,
-            ))
+            self._exec(
+                PostgresqlColumnType(
+                    table_name,
+                    column_name,
+                    type_,
+                    schema=schema,
+                    using=using,
+                    existing_type=existing_type,
+                    existing_server_default=existing_server_default,
+                    existing_nullable=existing_nullable,
+                )
+            )
 
         super(PostgresqlImpl, self).alter_column(
-            table_name, column_name,
+            table_name,
+            column_name,
             nullable=nullable,
             server_default=server_default,
             name=name,
@@ -112,58 +137,61 @@ class PostgresqlImpl(DefaultImpl):
             existing_server_default=existing_server_default,
             existing_nullable=existing_nullable,
             existing_autoincrement=existing_autoincrement,
-            **kw)
+            **kw
+        )
 
     def autogen_column_reflect(self, inspector, table, column_info):
-        if column_info.get('default') and \
-                isinstance(column_info['type'], (INTEGER, BIGINT)):
+        if column_info.get("default") and isinstance(
+            column_info["type"], (INTEGER, BIGINT)
+        ):
             seq_match = re.match(
-                r"nextval\('(.+?)'::regclass\)",
-                column_info['default'])
+                r"nextval\('(.+?)'::regclass\)", column_info["default"]
+            )
             if seq_match:
-                info = inspector.bind.execute(text(
-                    "select c.relname, a.attname "
-                    "from pg_class as c join pg_depend d on d.objid=c.oid and "
-                    "d.classid='pg_class'::regclass and "
-                    "d.refclassid='pg_class'::regclass "
-                    "join pg_class t on t.oid=d.refobjid "
-                    "join pg_attribute a on a.attrelid=t.oid and "
-                    "a.attnum=d.refobjsubid "
-                    "where c.relkind='S' and c.relname=:seqname"
-                ), seqname=seq_match.group(1)).first()
+                info = inspector.bind.execute(
+                    text(
+                        "select c.relname, a.attname "
+                        "from pg_class as c join "
+                        "pg_depend d on d.objid=c.oid and "
+                        "d.classid='pg_class'::regclass and "
+                        "d.refclassid='pg_class'::regclass "
+                        "join pg_class t on t.oid=d.refobjid "
+                        "join pg_attribute a on a.attrelid=t.oid and "
+                        "a.attnum=d.refobjsubid "
+                        "where c.relkind='S' and c.relname=:seqname"
+                    ),
+                    seqname=seq_match.group(1),
+                ).first()
                 if info:
                     seqname, colname = info
-                    if colname == column_info['name']:
+                    if colname == column_info["name"]:
                         log.info(
                             "Detected sequence named '%s' as "
                             "owned by integer column '%s(%s)', "
                             "assuming SERIAL and omitting",
-                            seqname, table.name, colname)
+                            seqname,
+                            table.name,
+                            colname,
+                        )
                         # sequence, and the owner is this column,
                         # its a SERIAL - whack it!
-                        del column_info['default']
+                        del column_info["default"]
 
-    def correct_for_autogen_constraints(self, conn_unique_constraints,
-                                        conn_indexes,
-                                        metadata_unique_constraints,
-                                        metadata_indexes):
+    def correct_for_autogen_constraints(
+        self,
+        conn_unique_constraints,
+        conn_indexes,
+        metadata_unique_constraints,
+        metadata_indexes,
+    ):
 
-        conn_uniques_by_name = dict(
-            (c.name, c) for c in conn_unique_constraints)
-        conn_indexes_by_name = dict(
-            (c.name, c) for c in conn_indexes)
+        conn_indexes_by_name = dict((c.name, c) for c in conn_indexes)
 
-        if not util.sqla_100:
-            doubled_constraints = set(
-                conn_indexes_by_name[name]
-                for name in set(conn_uniques_by_name).intersection(
-                    conn_indexes_by_name)
-            )
-        else:
-            doubled_constraints = set(
-                index for index in
-                conn_indexes if index.info.get('duplicates_constraint')
-            )
+        doubled_constraints = set(
+            index
+            for index in conn_indexes
+            if index.info.get("duplicates_constraint")
+        )
 
         for ix in doubled_constraints:
             conn_indexes.remove(ix)
@@ -187,37 +215,36 @@ class PostgresqlImpl(DefaultImpl):
         if not mod.startswith("sqlalchemy.dialects.postgresql"):
             return False
 
-        if hasattr(self, '_render_%s_type' % type_.__visit_name__):
-            meth = getattr(self, '_render_%s_type' % type_.__visit_name__)
+        if hasattr(self, "_render_%s_type" % type_.__visit_name__):
+            meth = getattr(self, "_render_%s_type" % type_.__visit_name__)
             return meth(type_, autogen_context)
 
         return False
 
     def _render_HSTORE_type(self, type_, autogen_context):
         return render._render_type_w_subtype(
-            type_, autogen_context, 'text_type', r'(.+?\(.*text_type=)'
+            type_, autogen_context, "text_type", r"(.+?\(.*text_type=)"
         )
 
     def _render_ARRAY_type(self, type_, autogen_context):
         return render._render_type_w_subtype(
-            type_, autogen_context, 'item_type', r'(.+?\()'
+            type_, autogen_context, "item_type", r"(.+?\()"
         )
 
     def _render_JSON_type(self, type_, autogen_context):
         return render._render_type_w_subtype(
-            type_, autogen_context, 'astext_type', r'(.+?\(.*astext_type=)'
+            type_, autogen_context, "astext_type", r"(.+?\(.*astext_type=)"
         )
 
     def _render_JSONB_type(self, type_, autogen_context):
         return render._render_type_w_subtype(
-            type_, autogen_context, 'astext_type', r'(.+?\(.*astext_type=)'
+            type_, autogen_context, "astext_type", r"(.+?\(.*astext_type=)"
         )
 
 
 class PostgresqlColumnType(AlterColumn):
-
     def __init__(self, name, column_name, type_, **kw):
-        using = kw.pop('using', None)
+        using = kw.pop("using", None)
         super(PostgresqlColumnType, self).__init__(name, column_name, **kw)
         self.type_ = sqltypes.to_instance(type_)
         self.using = using
@@ -227,7 +254,7 @@ class PostgresqlColumnType(AlterColumn):
 def visit_rename_table(element, compiler, **kw):
     return "%s RENAME TO %s" % (
         alter_table(compiler, element.table_name, element.schema),
-        format_table_name(compiler, element.new_table_name, None)
+        format_table_name(compiler, element.new_table_name, None),
     )
 
 
@@ -237,13 +264,32 @@ def visit_column_type(element, compiler, **kw):
         alter_table(compiler, element.table_name, element.schema),
         alter_column(compiler, element.column_name),
         "TYPE %s" % format_type(compiler, element.type_),
-        "USING %s" % element.using if element.using else ""
+        "USING %s" % element.using if element.using else "",
+    )
+
+
+@compiles(ColumnComment, "postgresql")
+def visit_column_comment(element, compiler, **kw):
+    ddl = "COMMENT ON COLUMN {table_name}.{column_name} IS {comment}"
+    comment = (
+        compiler.sql_compiler.render_literal_value(
+            element.comment, sqltypes.String()
+        )
+        if element.comment is not None
+        else "NULL"
+    )
+
+    return ddl.format(
+        table_name=element.table_name,
+        column_name=element.column_name,
+        comment=comment,
     )
 
 
 @Operations.register_operation("create_exclude_constraint")
 @BatchOperations.register_operation(
-    "create_exclude_constraint", "batch_create_exclude_constraint")
+    "create_exclude_constraint", "batch_create_exclude_constraint"
+)
 @ops.AddConstraintOp.register_add_constraint("exclude_constraint")
 class CreateExcludeConstraintOp(ops.AddConstraintOp):
     """Represent a create exclude constraint operation."""
@@ -251,9 +297,15 @@ class CreateExcludeConstraintOp(ops.AddConstraintOp):
     constraint_type = "exclude"
 
     def __init__(
-            self, constraint_name, table_name,
-            elements, where=None, schema=None,
-            _orig_constraint=None, **kw):
+        self,
+        constraint_name,
+        table_name,
+        elements,
+        where=None,
+        schema=None,
+        _orig_constraint=None,
+        **kw
+    ):
         self.constraint_name = constraint_name
         self.table_name = table_name
         self.elements = elements
@@ -275,13 +327,10 @@ class CreateExcludeConstraintOp(ops.AddConstraintOp):
             _orig_constraint=constraint,
             deferrable=constraint.deferrable,
             initially=constraint.initially,
-            using=constraint.using
+            using=constraint.using,
         )
 
     def to_constraint(self, migration_context=None):
-        if not util.sqla_100:
-            raise NotImplementedError(
-                "ExcludeConstraint not supported until SQLAlchemy 1.0")
         if self._orig_constraint is not None:
             return self._orig_constraint
         schema_obj = schemaobj.SchemaObjects(migration_context)
@@ -299,8 +348,8 @@ class CreateExcludeConstraintOp(ops.AddConstraintOp):
 
     @classmethod
     def create_exclude_constraint(
-            cls, operations,
-            constraint_name, table_name, *elements, **kw):
+        cls, operations, constraint_name, table_name, *elements, **kw
+    ):
         """Issue an alter to create an EXCLUDE constraint using the
         current migration context.
 
@@ -344,7 +393,8 @@ class CreateExcludeConstraintOp(ops.AddConstraintOp):
 
     @classmethod
     def batch_create_exclude_constraint(
-            cls, operations, constraint_name, *elements, **kw):
+        cls, operations, constraint_name, *elements, **kw
+    ):
         """Issue a "create exclude constraint" instruction using the
         current batch migration context.
 
@@ -358,28 +408,25 @@ class CreateExcludeConstraintOp(ops.AddConstraintOp):
             :meth:`.Operations.create_exclude_constraint`
 
         """
-        kw['schema'] = operations.impl.schema
+        kw["schema"] = operations.impl.schema
         op = cls(constraint_name, operations.impl.table_name, elements, **kw)
         return operations.invoke(op)
 
 
 @render.renderers.dispatch_for(CreateExcludeConstraintOp)
 def _add_exclude_constraint(autogen_context, op):
-    return _exclude_constraint(
-        op.to_constraint(),
-        autogen_context,
-        alter=True
+    return _exclude_constraint(op.to_constraint(), autogen_context, alter=True)
+
+
+@render._constraint_renderers.dispatch_for(ExcludeConstraint)
+def _render_inline_exclude_constraint(constraint, autogen_context):
+    rendered = render._user_defined_render(
+        "exclude", constraint, autogen_context
     )
+    if rendered is not False:
+        return rendered
 
-if util.sqla_100:
-    @render._constraint_renderers.dispatch_for(ExcludeConstraint)
-    def _render_inline_exclude_constraint(constraint, autogen_context):
-        rendered = render._user_defined_render(
-            "exclude", constraint, autogen_context)
-        if rendered is not False:
-            return rendered
-
-        return _exclude_constraint(constraint, autogen_context, False)
+    return _exclude_constraint(constraint, autogen_context, False)
 
 
 def _postgresql_autogenerate_prefix(autogen_context):
@@ -405,48 +452,54 @@ def _exclude_constraint(constraint, autogen_context, alter):
         opts.append(("schema", render._ident(constraint.table.schema)))
     if not alter and constraint.name:
         opts.append(
-            ("name",
-             render._render_gen_name(autogen_context, constraint.name)))
+            ("name", render._render_gen_name(autogen_context, constraint.name))
+        )
 
     if alter:
         args = [
-            repr(render._render_gen_name(
-                autogen_context, constraint.name))]
+            repr(render._render_gen_name(autogen_context, constraint.name))
+        ]
         if not has_batch:
             args += [repr(render._ident(constraint.table.name))]
-        args.extend([
-            "(%s, %r)" % (
-                _render_potential_column(sqltext, autogen_context),
-                opstring
-            )
-            for sqltext, name, opstring in constraint._render_exprs
-        ])
+        args.extend(
+            [
+                "(%s, %r)"
+                % (
+                    _render_potential_column(sqltext, autogen_context),
+                    opstring,
+                )
+                for sqltext, name, opstring in constraint._render_exprs
+            ]
+        )
         if constraint.where is not None:
             args.append(
-                "where=%s" % render._render_potential_expr(
-                    constraint.where, autogen_context)
+                "where=%s"
+                % render._render_potential_expr(
+                    constraint.where, autogen_context
+                )
             )
         args.extend(["%s=%r" % (k, v) for k, v in opts])
         return "%(prefix)screate_exclude_constraint(%(args)s)" % {
-            'prefix': render._alembic_autogenerate_prefix(autogen_context),
-            'args': ", ".join(args)
+            "prefix": render._alembic_autogenerate_prefix(autogen_context),
+            "args": ", ".join(args),
         }
     else:
         args = [
-            "(%s, %r)" % (
-                _render_potential_column(sqltext, autogen_context),
-                opstring
-            ) for sqltext, name, opstring in constraint._render_exprs
+            "(%s, %r)"
+            % (_render_potential_column(sqltext, autogen_context), opstring)
+            for sqltext, name, opstring in constraint._render_exprs
         ]
         if constraint.where is not None:
             args.append(
-                "where=%s" % render._render_potential_expr(
-                    constraint.where, autogen_context)
+                "where=%s"
+                % render._render_potential_expr(
+                    constraint.where, autogen_context
+                )
             )
         args.extend(["%s=%r" % (k, v) for k, v in opts])
         return "%(prefix)sExcludeConstraint(%(args)s)" % {
             "prefix": _postgresql_autogenerate_prefix(autogen_context),
-            "args": ", ".join(args)
+            "args": ", ".join(args),
         }
 
 
@@ -456,8 +509,10 @@ def _render_potential_column(value, autogen_context):
 
         return template % {
             "prefix": render._sqlalchemy_autogenerate_prefix(autogen_context),
-            "name": value.name
+            "name": value.name,
         }
 
     else:
-        return render._render_potential_expr(value, autogen_context, wrap_in_text=False)
+        return render._render_potential_expr(
+            value, autogen_context, wrap_in_text=False
+        )
